@@ -17,28 +17,23 @@
 #ifndef GLOBUS_DONT_DOCUMENT_INTERNAL
 /**
  * @file globus_callout.c
- * Globus Callout Infrastructure
+ * @brief Globus Callout Infrastructure
  * @author Sam Meder
- *
- * $RCSfile: globus_callout.c,v $
- * $Revision: 1.25.14.3 $
- * $Date: 2013/06/06 05:42:45 $
  */
+#endif /* GLOBUS_DONT_DOCUMENT_INTERNAL */
 
 #include "globus_common.h"
 #include "globus_callout_constants.h"
 #include "globus_i_callout.h"
 
-#ifndef BUILD_STATIC_ONLY
 #include <ltdl.h>
-#endif
 
 #include "version.h"
 
 #define GLOBUS_I_CALLOUT_HASH_SIZE 64
 #define GLOBUS_I_CALLOUT_LINEBUF 4096
 
-#ifdef WIN32
+#if defined(_WIN32) || defined(__CYGWIN__)
 #define MY_LIB_EXT ".dll"
 #else
 #define MY_LIB_EXT ".so"
@@ -50,7 +45,7 @@ globus_l_callout_library_table_element_free(
 
 static void
 globus_l_callout_data_free(
-    globus_i_callout_data_t *           data);
+    void *           data);
 
 
 static int globus_l_callout_activate(void);
@@ -121,12 +116,10 @@ globus_l_callout_activate(void)
         goto exit;
     }
 
-#ifndef BUILD_STATIC_ONLY
     if((result = lt_dlinit()) != 0)
     {
         goto exit;
     }
-#endif
 
     GLOBUS_I_CALLOUT_DEBUG_EXIT;
 
@@ -158,19 +151,11 @@ globus_l_callout_deactivate(void)
         fclose(globus_i_callout_debug_fstream);
     }
 
-#ifndef BUILD_STATIC_ONLY
     result = lt_dlexit();
-#endif
 
     return result;
 }
 
-#endif
-
-/**
- * @name Initialize Handle
- */
-/*@{*/
 /**
  * Initialize a Globus Callout Handle
  * @ingroup globus_callout_handle
@@ -178,9 +163,9 @@ globus_l_callout_deactivate(void)
  * @param handle
  *        Pointer to the handle that is to be initialized
  * @return
- *        GLOBUS_SUCCESS if successful
- *        A Globus error object on failure:
- *            GLOBUS_CALLOUT_ERROR_WITH_HASHTABLE
+ *    This function returns GLOBUS_SUCCESS or a globus_result_t referring
+ *    to an error object of one of the  following types
+ * @retval GLOBUS_CALLOUT_ERROR_WITH_HASHTABLE Hashtable initialization failed
  */
 globus_result_t
 globus_callout_handle_init(
@@ -235,12 +220,7 @@ globus_callout_handle_init(
     
     return result;
 }/*globus_callout_handle_init*/
-/*@}*/
 
-/**
- * @name Destroy Handle
- */
-/*@{*/
 /**
  * Destroy a Globus Callout Handle
  * @ingroup globus_callout_handle
@@ -275,39 +255,36 @@ globus_callout_handle_destroy(
 
     return result;
 }/*globus_callout_handle_destroy*/
-/*@}*/
 
-/**
- * @name Configure Callouts
- */
-/*@{*/
 /**
  * Read callout configuration from file.
  * @ingroup globus_callout_config
  *
  * This function read a configuration file with the following format:
- *    - Anything after a '#' is assumed to be a comment
+ *    - Anything after a <i>#</i> is assumed to be a comment
  *    - Blanks lines are ignored
  *    - Lines specifying callouts have the format
- *      abstract type           library         symbol
- *      where "abstract type" denotes the type of callout,
- *      e.g. globus_gram_jobmanager_authz, "library" denotes the library the
- *      callout can be found in and "symbol" denotes the function name of the
- *      callout. The library argument can be specified in two forms, libfoo or
- *      libfoo_<flavor>. When using the former version the current flavor will
- *      automatically be added to the library name. 
+ @verbatim
+        abstract type           library         symbol
+ @endverbatim
+ *      where <i>abstract type</i> denotes the type of callout,
+ *      e.g. globus_gram_jobmanager_authz, <i>library</i> denotes the library
+ *      the callout can be found in and <i>symbol</i> denotes the function name
+ *      of the callout. The library argument can be specified in two forms,
+ *      libfoo or libfoo_<i>flavor</i>. When using the former version the
+ *      current flavor will automatically be added to the library name if
+ *      needed. 
  *
  * @param handle
  *        The handle that is to be configured
  * @param filename
  *        The file to read configuration from
  * @return
- *        GLOBUS_SUCCESS
- *        A Globus error object on failure:
- *            GLOBUS_CALLOUT_ERROR_OPENING_CONF_FILE
- *            GLOBUS_CALLOUT_ERROR_PARSING_CONF_FILE
- *            GLOBUS_CALLOUT_ERROR_WITH_HASHTABLE
- *            GLOBUS_CALLOUT_ERROR_OUT_OF_MEMORY
+ *    This function returns GLOBUS_SUCCESS or a globus_result_t referring
+ *    to an error object of one of the  following types
+ * @retval GLOBUS_CALLOUT_ERROR_OPENING_CONF_FILE Error opening filename
+ * @retval GLOBUS_CALLOUT_ERROR_PARSING_CONF_FILE Error parsing file
+ * @retval GLOBUS_CALLOUT_ERROR_OUT_OF_MEMORY Out of memory
  */
 globus_result_t
 globus_callout_read_config(
@@ -572,14 +549,6 @@ globus_callout_read_config(
             }
             existing_datum->next = datum;
         }
-        else if(rc < 0)
-        {
-            GLOBUS_CALLOUT_ERROR_RESULT(
-                result,
-                GLOBUS_CALLOUT_ERROR_WITH_HASHTABLE,
-                ("globus_hashtable_insert retuned %d", rc));
-            goto error_exit;
-        }
     }
 
     fclose(conf_file);
@@ -618,10 +587,9 @@ globus_callout_read_config(
  * @param symbol
  *        The symbol (ie function name) for the callout
  * @return
- *        GLOBUS_SUCCESS
- *        A Globus error object on failure:
- *            GLOBUS_CALLOUT_ERROR_WITH_HASHTABLE
- *            GLOBUS_CALLOUT_ERROR_OUT_OF_MEMORY
+ *    This function returns GLOBUS_SUCCESS or a globus_result_t referring
+ *    to an error object of one of the following types
+ * @retval GLOBUS_CALLOUT_ERROR_OUT_OF_MEMORY Out of memory
  */
 globus_result_t
 globus_callout_register(
@@ -653,6 +621,7 @@ globus_callout_register(
     }
     
     memset(datum,'\0',sizeof(globus_i_callout_data_t));
+    datum->mandatory = GLOBUS_TRUE;
 
     if((flavor_start = strrchr(library,'_')) &&
        (strstr(flavor_start, "32") || strstr(flavor_start, "64")))
@@ -707,14 +676,6 @@ globus_callout_register(
         }
         existing_datum->next = datum;
     }
-    else if(rc < 0)
-    {
-        GLOBUS_CALLOUT_ERROR_RESULT(
-            result,
-            GLOBUS_CALLOUT_ERROR_WITH_HASHTABLE,
-            ("globus_hashtable_insert retuned %d", rc));
-        goto error_exit;
-    }
     
     GLOBUS_I_CALLOUT_DEBUG_EXIT;
 
@@ -731,13 +692,7 @@ globus_callout_register(
  
     return result;
 }/*globus_callout_register*/
-/*@}*/
 
-
-/**
- * @name Invoking Callouts
- */
-/*@{*/
 /**
  * Call a callout of specified abstract type
  * @ingroup globus_callout_call
@@ -752,13 +707,13 @@ globus_callout_register(
  * @param type
  *        The abstract type of the callout that is to be invoked
  * @return
- *        GLOBUS_SUCCESS
- *        A Globus error object on failure:
- *            GLOBUS_CALLOUT_ERROR_TYPE_NOT_REGISTERED
- *            GLOBUS_CALLOUT_ERROR_CALLOUT_ERROR
- *            GLOBUS_CALLOUT_ERROR_WITH_DL
- *            GLOBUS_CALLOUT_ERROR_WITH_HASHTABLE
- *            GLOBUS_CALLOUT_ERROR_OUT_OF_MEMORY
+ *    This function returns GLOBUS_SUCCESS or a globus_result_t referring
+ *    to an error object of one of the following types
+ * @retval GLOBUS_CALLOUT_ERROR_TYPE_NOT_REGISTERED Callout type not registered
+ * @retval GLOBUS_CALLOUT_ERROR_CALLOUT_ERROR Callout function error
+ * @retval GLOBUS_CALLOUT_ERROR_WITH_DL Error with dlopen or dlsym
+ * @retval GLOBUS_CALLOUT_ERROR_WITH_HASHTABLE Error caching dlopen handle
+ * @retval GLOBUS_CALLOUT_ERROR_OUT_OF_MEMORY Out of memory
  */
 globus_result_t
 globus_callout_call_type(
@@ -767,16 +722,12 @@ globus_callout_call_type(
     ...)
 {
     globus_i_callout_data_t *           current_datum;
-#ifdef BUILD_STATIC_ONLY
-    void *                              function;
-#else
     lt_ptr                              function;
     lt_dlhandle *                       dlhandle;
-#endif
     globus_result_t                     result = GLOBUS_SUCCESS;
     va_list                             ap;
     int                                 rc;
-    char *                              dlerror;
+    const char *                        dlerror;
     char *                              flavor_start;
     char *                              file;
     char                                library[1024];
@@ -811,15 +762,6 @@ globus_callout_call_type(
     
     do
     {
-#ifdef BUILD_STATIC_ONLY
-        GLOBUS_CALLOUT_ERROR_RESULT(
-            result,
-            GLOBUS_CALLOUT_ERROR_WITH_DL,
-            ("couldn't dlopen %s: %s\n",
-             current_datum->file,
-             "(null)"));
-        goto exit;
-#else
         dlhandle = globus_hashtable_lookup(&handle->library_htable,
                                            current_datum->file);
 
@@ -856,7 +798,7 @@ globus_callout_call_type(
             *dlhandle = lt_dlopenext(current_datum->file);
             if(*dlhandle == NULL)
             {
-                /* older libtools dont search the extensions correctly */
+                /* older libtools don't search the extensions correctly */
                 snprintf(library, 1024, "%s" MY_LIB_EXT, current_datum->file);
                 library[1023] = 0;
                 *dlhandle = lt_dlopenext(library);
@@ -877,7 +819,7 @@ globus_callout_call_type(
                     *dlhandle = lt_dlopenext(file);
                     if(*dlhandle == NULL)
                     {
-                        /* older libtools dont search the extensions correctly */
+                        /* older libtools don't search the extensions correctly */
                         snprintf(library, 1024, "%s" MY_LIB_EXT, file);
                         library[1023] = 0;
                         *dlhandle = lt_dlopenext(library);
@@ -924,7 +866,7 @@ globus_callout_call_type(
                 save_env[i] = current_datum->env_args[i];
                 save_env[i+1] = 
                     globus_libc_strdup(getenv(current_datum->env_args[i]));
-                setenv(current_datum->env_args[i], current_datum->env_args[i+1], 1);
+                globus_libc_setenv(current_datum->env_args[i], current_datum->env_args[i+1], 1);
                 i += 2;
             }
             save_env[i] = NULL;
@@ -943,11 +885,11 @@ globus_callout_call_type(
             {
                 if(save_env[i+1] == NULL)
                 {
-                    unsetenv(save_env[i]);
+                    globus_libc_unsetenv(save_env[i]);
                 }
                 else
                 {
-                    setenv(save_env[i], save_env[i+1], 1);
+                    globus_libc_setenv(save_env[i], save_env[i+1], 1);
                     globus_free(save_env[i+1]);
                 }
                                 
@@ -994,7 +936,6 @@ globus_callout_call_type(
         }
 
         current_datum = current_datum->next;
-#endif
     }
     while(current_datum);
     
@@ -1002,16 +943,16 @@ globus_callout_call_type(
     GLOBUS_I_CALLOUT_DEBUG_EXIT;
     return result;
 }/*globus_callout_call_type*/
-/*@}*/
 
 #ifndef GLOBUS_DONT_DOCUMENT_INTERNAL
 
 static void
 globus_l_callout_data_free(
-    globus_i_callout_data_t *           data)
+    void *           datum)
 {
     static char *                       _function_name_ =
         "globus_l_callout_data_free";
+    globus_i_callout_data_t *           data = datum;
     GLOBUS_I_CALLOUT_DEBUG_ENTER;
 
     if(data != NULL)
@@ -1054,15 +995,12 @@ static void
 globus_l_callout_library_table_element_free(
     void *                              element)
 {
-#ifndef BUILD_STATIC_ONLY
     lt_dlhandle *                       dlhandle;
-#endif
     globus_result_t                     result;
     static char *                       _function_name_ =
         "globus_l_callout_library_table_element_free";
     GLOBUS_I_CALLOUT_DEBUG_ENTER;
 
-#ifndef BUILD_STATIC_ONLY
     dlhandle = (lt_dlhandle *) element;
 
     if(dlhandle != NULL)
@@ -1080,10 +1018,9 @@ globus_l_callout_library_table_element_free(
 
         free(dlhandle);
     }
-#endif
     
     GLOBUS_I_CALLOUT_DEBUG_EXIT;
     return;
 }
 
-#endif
+#endif /* GLOBUS_DONT_DOCUMENT_INTERNAL */
